var l=Object.defineProperty;var i=(n,r)=>l(n,"name",{value:r,configurable:!0});import{r as a}from"./get-pipe-path-BHW2eJdv.mjs";import{constants as f}from"node:os";import{isMainThread as d}from"node:worker_threads";import{c as u}from"./client-BQVF1NaW.mjs";import"./suppress-warnings.mjs";import"module";import"node:path";import"./temporary-directory-CwHp0_NW.mjs";import"node:net";const m=i((n,r)=>{const o=new Map;for(const e of n){const t=i(s=>{r(s),process.listenerCount(e)===0&&process.exit(128+f.signals[e])},"hiddenHandler");process.on(e,t),o.set(e,t)}const{listenerCount:c,listeners:p}=process;process.listenerCount=function(e){let t=Reflect.apply(c,this,arguments);return n.includes(e)&&(t-=1),t},process.listeners=function(e){const t=Reflect.apply(p,this,arguments);return n.includes(e)?t.filter(s=>s!==o.get(e)):t}},"bindHiddenSignalsHandler");d&&(a("./cjs/index.cjs"),(async()=>{const n=await u;n&&m(["SIGINT","SIGTERM"],r=>{n({type:"signal",signal:r})})})());
