import type { ParamAssocArray } from './node';
export type ReplacementMap = number[];
export declare class Trie {
             
    insert(path: string, index: number, pathErrorCheckOnly: boolean): ParamAssocArray;
    buildRegExp(): [RegExp, ReplacementMap, ReplacementMap];
}
