import type { Child, FC, PropsWithChildren } from '../';
import type { ErrorHandler, FallbackRender } from '../components';
export declare const ErrorBoundary: FC<PropsWithChildren<{
    fallback?: Child;
    fallbackRender?: FallbackRender;
    onError?: ErrorHandler;
}>>;
export declare const Suspense: FC<PropsWithChildren<{
    fallback: any;
}>>;
