// src/adapter/cloudflare-workers/index.ts
import { serveStatic } from "./serve-static-module.js";
import { upgradeWebSocket } from "./websocket.js";
import { getConnInfo } from "./conninfo.js";
export {
  getConnInfo,
  serveStatic,
  upgradeWebSocket
};
