export { c as createManualModuleSource, h as hoistMocks } from './hoistMocks.d-w2ILr1dG.js';
export { automockModule } from './automock.js';
import 'magic-string';

declare function initSyntaxLexers(): Promise<void>;
declare function collectModuleExports(filename: string, code: string, format: "module" | "commonjs", exports?: string[]): string[];

export { collectModuleExports, initSyntaxLexers };
