// src/adapter/cloudflare-pages/index.ts
import { handle, handleMiddleware, serveStatic } from "./handler.js";
import { getConnInfo } from "./conninfo.js";
export {
  getConnInfo,
  handle,
  handleMiddleware,
  serveStatic
};
