export type KVAssetOptions = {
    manifest?: object | string;
    namespace?: unknown;
};
export declare const getContentFromKVAsset: (path: string, options?: KVAssetOptions) => Promise<ReadableStream | null>;
