export type { ContentSecurityPolicyOptionHandler } from './secure-headers';
export { NONCE, secureHeaders } from './secure-headers';
import type { SecureHeadersVariables } from './secure-headers';
export type { SecureHeadersVariables };
declare module '../..' {
    interface ContextVariableMap extends SecureHeadersVariables {
    }
}
