import 'node:fs';
import 'node:url';
import 'magic-string';
export { a as automockModule } from './chunk-automock.js';
import 'estree-walker';
import 'node:module';
import 'node:path';
import './chunk-helpers.js';
