// src/adapter/aws-lambda/index.ts
import { handle, streamHandle, defaultIsContentTypeBinary } from "./handler.js";
import { getConnInfo } from "./conninfo.js";
export {
  defaultIsContentTypeBinary,
  getConnInfo,
  handle,
  streamHandle
};
