import { c as createCLI } from './chunks/cac.CWGDZnXT.js';
import '@vitest/utils/helpers';
import 'events';
import 'pathe';
import 'tinyrainbow';
import './chunks/constants.CPYnjOGj.js';
import './chunks/index.DXMFO5MJ.js';
import 'node:fs';
import 'node:perf_hooks';
import '@vitest/runner/utils';
import '@vitest/utils/source-map';
import './chunks/env.D4Lgay0q.js';
import 'std-env';
import 'node:console';
import 'node:stream';
import 'node:util';
import '@vitest/utils/display';
import 'node:os';
import 'tinyexec';
import './path.js';
import 'node:path';
import 'node:url';
import 'vite';
import '@vitest/utils/offset';
import 'node:fs/promises';
import 'node:module';
import 'node:assert';

createCLI().parse();
