import { resolve } from 'node:path';
import url from 'node:url';

const rootDir = resolve(url.fileURLToPath(import.meta.url), "../../");
const distDir = resolve(url.fileURLToPath(import.meta.url), "../../dist");

export { distDir, rootDir };
