import type { Hono } from '../hono';
import type { UnionToIntersection } from '../utils/types';
import type { Client, ClientRequestOptions } from './types';
export declare const hc: <T extends Hono<any, any, any>, Prefix extends string = string>(baseUrl: Prefix, options?: ClientRequestOptions) => UnionToIntersection<Client<T, Prefix>>;
