/**
 * @module
 * Lambda@Edge Adapter for Hono.
 */
export { handle } from './handler';
export { getConnInfo } from './conninfo';
export type { Callback, CloudFrontConfig, CloudFrontRequest, CloudFrontResponse, CloudFrontEdgeEvent, } from './handler';
