/**
 * @module
 * The HTTP Client for Hono.
 */
export { hc } from './client';
export { parseResponse, DetailedError } from './utils';
export type { InferResponseType, InferRequestType, Fetch, ClientRequestOptions, ClientRequest, ClientResponse, ApplyGlobalResponse, } from './types';
