import type { ServeStaticOptions } from '../../middleware/serve-static';
import type { Env, MiddlewareHandler } from '../../types';
export declare const serveStatic: <E extends Env = Env>(options: ServeStaticOptions<E>) => MiddlewareHandler;
