import { w as workerInit } from '../chunks/init-threads.-2OUl4Nn.js';
import { s as setupVmWorker, r as runVmTests } from '../chunks/vm.Dh2rTtmP.js';
import 'node:worker_threads';
import '../chunks/init.DICorXCo.js';
import 'node:fs';
import 'node:module';
import 'node:url';
import 'pathe';
import 'vite/module-runner';
import '../chunks/startVitestModuleRunner.C3ZR-4J3.js';
import '@vitest/utils/helpers';
import '../chunks/modules.BJuCwlRJ.js';
import '../chunks/utils.BX5Fg8C4.js';
import '@vitest/utils/timers';
import '../path.js';
import 'node:path';
import '../module-evaluator.js';
import 'node:vm';
import '../chunks/traces.CCmnQaNT.js';
import '@vitest/mocker';
import '@vitest/mocker/redirect';
import '../chunks/index.EY6TCHpo.js';
import 'node:console';
import '@vitest/utils/serialize';
import '@vitest/utils/error';
import '../chunks/rpc.MzXet3jl.js';
import '../chunks/index.Chj8NDwU.js';
import '@vitest/utils/source-map';
import '../chunks/inspector.CvyFGlXm.js';
import '../chunks/evaluatedModules.Dg1zASAC.js';
import '../chunks/console.3WNpx0tS.js';
import 'node:stream';
import 'tinyrainbow';
import '@vitest/utils/resolver';
import '@vitest/utils/constants';

workerInit({
	runTests: runVmTests,
	setup: setupVmWorker
});
