export { r as runBaseTests, s as setupEnvironment } from './chunks/base.DM0-RqVb.js';
export { i as init } from './chunks/init.DICorXCo.js';
import 'node:vm';
import '@vitest/spy';
import './chunks/index.DGNSnENe.js';
import '@vitest/expect';
import 'node:async_hooks';
import './chunks/setup-common.B41N_kPE.js';
import './chunks/coverage.D_JHT54q.js';
import '@vitest/snapshot';
import '@vitest/utils/timers';
import './chunks/utils.BX5Fg8C4.js';
import './chunks/rpc.MzXet3jl.js';
import './chunks/index.Chj8NDwU.js';
import './chunks/test.CTcmp4Su.js';
import '@vitest/runner';
import '@vitest/utils/helpers';
import './chunks/benchmark.D0SlKNbZ.js';
import '@vitest/runner/utils';
import '@vitest/utils/error';
import 'pathe';
import '@vitest/utils/offset';
import '@vitest/utils/source-map';
import './chunks/_commonjsHelpers.D26ty3Ew.js';
import './chunks/nativeModuleRunner.BIakptoF.js';
import 'node:url';
import './chunks/index.BCY_7LL2.js';
import 'node:fs';
import 'node:module';
import 'node:path';
import 'node:process';
import 'node:fs/promises';
import 'node:assert';
import 'node:v8';
import 'node:util';
import 'vite/module-runner';
import './chunks/traces.CCmnQaNT.js';
import './chunks/evaluatedModules.Dg1zASAC.js';
import './chunks/startVitestModuleRunner.C3ZR-4J3.js';
import './chunks/modules.BJuCwlRJ.js';
import './path.js';
import './module-evaluator.js';
import '@vitest/mocker';
import '@vitest/mocker/redirect';
import 'node:perf_hooks';
import './chunks/inspector.CvyFGlXm.js';
import 'node:timers';
import 'node:timers/promises';
import '@vitest/utils/constants';
import './chunks/index.DlDSLQD3.js';
import 'expect-type';
import './chunks/index.EY6TCHpo.js';
import 'node:console';
import '@vitest/utils/serialize';
