"use strict";var q=Object.defineProperty;var r=(i,u)=>q(i,"name",{value:u,configurable:!0});var t=require("./get-pipe-path-BoR10qr8.cjs"),e=require("./esm/index.cjs");require("module"),require("node:path"),require("./temporary-directory-B83uKxJF.cjs"),require("node:os"),require("node:worker_threads"),require("./node-features-roYmp9jK.cjs"),require("./register-2sWVXuRQ.cjs"),require("node:module"),require("./register-D46fvsV_.cjs"),require("node:url"),require("get-tsconfig"),require("node:fs"),require("./index-gckBtVBf.cjs"),require("esbuild"),require("node:crypto"),require("./client-D6NvIMSC.cjs"),require("node:net"),require("node:util"),require("./index-BWFBUo6r.cjs"),require("./require-D4F1Lv60.cjs"),require("node:fs/promises"),t.require("./cjs/index.cjs"),exports.globalPreload=e.globalPreload,exports.initialize=e.initialize,Object.defineProperty(exports,"load",{enumerable:!0,get:r(function(){return e.load},"get")}),Object.defineProperty(exports,"resolve",{enumerable:!0,get:r(function(){return e.resolve},"get")});
