import { VitestModuleEvaluator } from './module-evaluator.js';
import { V as VITEST_VM_CONTEXT_SYMBOL, s as startVitestModuleRunner, a as VitestModuleRunner } from './chunks/startVitestModuleRunner.C3ZR-4J3.js';
import { g as getWorkerState } from './chunks/utils.BX5Fg8C4.js';
export { e as builtinEnvironments, p as populateGlobal } from './chunks/index.EY6TCHpo.js';
export { VitestNodeSnapshotEnvironment as VitestSnapshotEnvironment } from './chunks/node.COQbm6gK.js';
import 'node:module';
import 'node:url';
import 'node:vm';
import 'vite/module-runner';
import './chunks/traces.CCmnQaNT.js';
import 'node:fs';
import '@vitest/utils/helpers';
import './chunks/modules.BJuCwlRJ.js';
import 'pathe';
import './path.js';
import 'node:path';
import '@vitest/mocker';
import '@vitest/mocker/redirect';
import '@vitest/utils/timers';
import 'node:console';
import '@vitest/snapshot/environment';

/**
* @internal
*/
const __INTERNAL = {
	VitestModuleEvaluator,
	VitestModuleRunner,
	startVitestModuleRunner,
	VITEST_VM_CONTEXT_SYMBOL,
	getWorkerState
};
// #endregion

export { __INTERNAL };
