/**
 * @module
 * Cloudflare Pages Adapter for Hono.
 */
export { handle, handleMiddleware, serveStatic } from './handler';
export { getConnInfo } from './conninfo';
export type { EventContext } from './handler';
