import { EvaluatedModules } from 'vite/module-runner';

declare class VitestEvaluatedModules extends EvaluatedModules {
	getModuleSourceMapById(id: string): any;
}

export { VitestEvaluatedModules as V };
