"use strict";var a=Object.defineProperty;var i=(r,s)=>a(r,"name",{value:s,configurable:!0});var l=require("./get-pipe-path-BoR10qr8.cjs"),d=require("node:os"),p=require("node:worker_threads"),f=require("./client-D6NvIMSC.cjs");require("./suppress-warnings.cjs"),require("module"),require("node:path"),require("./temporary-directory-B83uKxJF.cjs"),require("node:net");const q=i((r,s)=>{const o=new Map;for(const e of r){const n=i(t=>{s(t),process.listenerCount(e)===0&&process.exit(128+d.constants.signals[e])},"hiddenHandler");process.on(e,n),o.set(e,n)}const{listenerCount:c,listeners:u}=process;process.listenerCount=function(e){let n=Reflect.apply(c,this,arguments);return r.includes(e)&&(n-=1),n},process.listeners=function(e){const n=Reflect.apply(u,this,arguments);return r.includes(e)?n.filter(t=>t!==o.get(e)):n}},"bindHiddenSignalsHandler");p.isMainThread&&(l.require("./cjs/index.cjs"),(async()=>{const r=await f.connectingToServer;r&&q(["SIGINT","SIGTERM"],s=>{r({type:"signal",signal:s})})})());
