import type { JwtVariables } from './jwt';
export type { JwtVariables };
export { jwt, verifyWithJwks, verify, decode, sign } from './jwt';
export { AlgorithmTypes } from '../../utils/jwt/jwa';
declare module '../..' {
    interface ContextVariableMap extends JwtVariables<unknown> {
    }
}
