/**
 * @module
 * FilePath utility.
 */
type FilePathOptions = {
    filename: string;
    root?: string;
    defaultDocument?: string;
};
export declare const getFilePath: (options: FilePathOptions) => string | undefined;
export declare const getFilePathWithoutDefaultDocument: (options: Omit<FilePathOptions, "defaultDocument">) => string | undefined;
export {};
