export declare const DOM_RENDERER: unique symbol;
export declare const DOM_ERROR_HANDLER: unique symbol;
export declare const DOM_STASH: unique symbol;
export declare const DOM_INTERNAL_TAG: unique symbol;
export declare const DOM_MEMO: unique symbol;
export declare const PERMALINK: unique symbol;
