export { builtinEnvironments, populateGlobal } from './environments.js';
export { VitestSnapshotEnvironment } from './snapshot.js';
export { E as Environment, a as EnvironmentReturn, V as VmEnvironmentReturn } from './chunks/environment.d.CrsxCzP1.js';
export { VitestRunner, VitestRunnerConfig } from '@vitest/runner';
export { SnapshotEnvironment } from '@vitest/snapshot/environment';
import '@vitest/utils';
