export { e as builtinEnvironments, p as populateGlobal } from './chunks/index.EY6TCHpo.js';
import 'node:url';
import 'node:console';

console.warn("Importing from \"vitest/environments\" is deprecated since Vitest 4.1. Please use \"vitest/runtime\" instead.");
