/**
 * @module
 * This module provides the `hono/jsx/dom` dev runtime.
 */
import type { JSXNode, Props } from '../base';
export type { JSX } from '../base';
export declare const jsxDEV: (tag: string | Function, props: Props, key?: string) => JSXNode;
export declare const Fragment: (props: Record<string, unknown>) => JSXNode;
