import type { RequestIdVariables } from './request-id';
export type { RequestIdVariables };
export { requestId } from './request-id';
declare module '../..' {
    interface ContextVariableMap extends RequestIdVariables {
    }
}
